package org.example.country.report;

import java.util.List;

import org.apache.log4j.Logger;
import org.example.country.report.actions.CountryDBReader;
import org.example.country.report.actions.CountryDBReaderImpl;
import org.example.country.report.actions.PDFBuilder;
import org.example.country.report.actions.PDFBuilderImpl;
import org.example.country.report.actions.ConfigurationManager;
import org.example.country.report.actions.ConfigurationManagerImpl;
import org.example.country.report.entities.Country;

public class CountryReportAppl
{
	private static final String PROPERTY_FILE = "country_report.properties";
	private static final Logger LOG = Logger.getLogger(CountryReportAppl.class); 

	public static void main(String[] args)
	{
		LOG.info("Starting CountryReportAppl");
		
		LOG.debug("Creating PropertyManager");
		LOG.info("Property file: " + PROPERTY_FILE);		
		ConfigurationManager configManager = new ConfigurationManagerImpl();	
		LOG.debug("Loading " + PROPERTY_FILE);
		configManager.load(PROPERTY_FILE);
		
		LOG.debug("Creating PDFWriter");
		PDFBuilder pdfWriter = new PDFBuilderImpl();
		pdfWriter.initialize(configManager);
		pdfWriter.outputHeader();
		
		LOG.debug("Creating CountryDBReader");
		CountryDBReader dbReader = new CountryDBReaderImpl();
		dbReader.connect(configManager);
		
		LOG.debug("Reading Continents");
		List<String> continents = dbReader.findContinents();
		LOG.trace("Continents found: " + continents);
		
		for (String currentContinent : continents)
		{
			LOG.debug("Reading Countries for: " + currentContinent);
			List<Country> countries = dbReader.findCountriesByContinent(currentContinent);
			LOG.trace("Countries found: " + countries);
			
			LOG.debug("Writing to PDF");
			pdfWriter.outputCounties(currentContinent, countries);
		}
		
		LOG.debug("closing DBReader");
		dbReader.close();
		LOG.debug("closing PDFWriter");
		pdfWriter.complete();
		
		System.exit(0);
	}
}
