package org.example.country.report.actions;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.example.country.report.entities.Country;

public class CountryDBReaderImpl implements CountryDBReader
{

	private static final Logger LOG = Logger.getLogger(CountryDBReaderImpl.class);

	private Connection conn = null;
	private PreparedStatement continentPSMT = null;
	private String findContinentsSQL = "SELECT DISTINCT(continent) FROM jearl.country";

	private PreparedStatement countryPSMT = null;
	private String findContriesByContinentSQL = 
	  "SELECT country.code as Code, country.name as Name, " +
	  "country.continent as Continent, city.name as Capital " +
	  "FROM jearl.country AS country JOIN jearl.city AS city ON country.capital = city.id " +
	  "WHERE country.continent = ? " +
	  "ORDER BY Name";

	@Override
	public void connect(ConfigurationManager configManager)
	{
		LOG.debug("Connecting to database");
		String driver = configManager.getProperty("db.driver");
		String url = configManager.getProperty("db.url");
		String user = configManager.getProperty("db.user");
		String password = configManager.getProperty("db.password");
		LOG.trace("URL: " + url);

		try
		{
			LOG.debug("Loading driver: " + driver);
			Class.forName(driver);

			LOG.debug("Connecting to DB2");
			conn = DriverManager.getConnection(url, user, password);
			continentPSMT = conn.prepareStatement(findContinentsSQL);
			countryPSMT = conn.prepareStatement(findContriesByContinentSQL);
		} 
		catch (ClassNotFoundException e)
        {
            LOG.error("Failed to load driver");
            LOG.error(e.getMessage());
            System.exit(8);
        }
        catch (SQLException e)
        {
        	LOG.error("Problem connecting to DB");
            LOG.error(getError(e));
            System.exit(8);
        }
	}

	@Override
	public List<String> findContinents()
	{
		LOG.debug("Starting findContinents");
        List<String> results = new ArrayList<String>();
        ResultSet rset = null;
        
        try
        {
        	LOG.debug("executing the SQL");
            rset = continentPSMT.executeQuery();
            while(rset.next())
            {
                results.add(rset.getString(1));
            }
            rset.close();
        }
        catch (SQLException e)
        {
        	LOG.error(getError(e));
            System.exit(8);
        }
        return results;
	}

	@Override
	public List<Country> findCountriesByContinent(String currentContinent)
	{
		LOG.debug("Starting findCountriesByContinent: " + currentContinent);
        List<Country> results = new ArrayList<Country>();
        ResultSet rset = null;
        
        try
        {
        	LOG.debug("executing the SQL");
        	LOG.trace("Continent: " + currentContinent);
        	countryPSMT.setString(1, currentContinent);
            rset = countryPSMT.executeQuery();
            while(rset.next())
            {
                results.add(buildCountry(rset));
            }
       
            rset.close();
        }
        catch (SQLException e)
        {
        	LOG.error(getError(e));
            System.exit(8);
        }
        return results;
	}

	@Override
	public void close()
	{
		LOG.debug("closing database connection");
		try
		{
			continentPSMT.close();
			countryPSMT.close(); 
			conn.close();
		} 
		catch (SQLException e)
		{
			LOG.error(getError(e));
            System.exit(8);
		}

	}
	
	 private Country buildCountry(ResultSet results)
			 throws SQLException
	 {
		 try
		 {	 
			 Country record = new Country();
			 record.setCode(results.getString("Code"));
			 record.setName(results.getString("Name"));
			 record.setCapital(results.getString("Capital"));
			 record.setContinent(results.getString("Continent"));
			 LOG.trace("Building Country Entity " + record.toString());

			 return record;
		 }
		 catch (SQLException sqle)
		 {
			 LOG.error(getError(sqle));
			 throw sqle;
		 }
	 }

	private String getError(SQLException oops)
	{
		return "Failed to execute SQL," + " Error Code: " + oops.getErrorCode()
				+ " SQLState: " + oops.getSQLState() + " Message: "
				+ oops.getMessage();
	}
}
