package org.example.country.report.entities;

import static org.junit.Assert.*;

import org.apache.commons.lang.RandomStringUtils;
import org.junit.Before;
import org.junit.Test;

public class CountryTest 
{
	private Country testCountry;

	@Before
	public void setup() 
	{
		testCountry = new Country();
	}
	
	@Test 
	public void testInitialCode() 
	{
		String expected = "ABW";
		String actual = testCountry.getCode();
		assertEquals(expected, actual);
	}
	
	@Test 
	public void testInitialName() 
	{
		String expected = "Aruba";
		String actual = testCountry.getName();
		assertEquals(expected, actual);
	}
	
	@Test 
	public void testInitialCapital() 
	{
		String expected = "Oranjestad";
		String actual = testCountry.getCapital();
		assertEquals(expected, actual);
	}
	
	@Test 
	public void testInitialContinent() 
	{
		String expected = "North America";
		String actual = testCountry.getContinent();
		assertEquals(expected, actual);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testNullCode() 
	{
		testCountry.setCode(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testEmptyCode() 
	{
		testCountry.setCode("  ");
	}

	@Test(expected=IllegalArgumentException.class)
	public void testTooShortCode() 
	{
		String tooShort = "AB";
		testCountry.setCode(tooShort);
	}
	
	@Test(expected=IllegalArgumentException.class)
	public void testTooLongCode() 
	{
		String tooLong = "ABWX";
		testCountry.setCode(tooLong);
	}
	
	@Test 
	public void testValidCode() 
	{
		String[] values = {"USA", "GRB", "ABC", "GRC"};
		for (String value : values)
		{
			testCountry.setCode(value);
		}
	}
	
	@Test(expected = IllegalArgumentException.class)
	public void testNullName() {
		testCountry.setName(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testEmptyName() 
	{
		testCountry.setName("  ");
	}

	@Test(expected=IllegalArgumentException.class)
	public void testTooLongName() 
	{
		String tooLong = RandomStringUtils.randomAlphanumeric(53);
		testCountry.setName(tooLong);
	}
	
	@Test 
	public void testValidName() 
	{
		String[] values = {"Canada", "Japan", "Greece", "Spain"};
		for (String value : values)
		{
			testCountry.setName(value);
		}
	}
	
	@Test(expected = IllegalArgumentException.class)
	public void testNullCapital() 
	{
		testCountry.setCapital(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testEmptyCapital() 
	{
		testCountry.setCapital("  ");
	}

	@Test(expected=IllegalArgumentException.class)
	public void testTooLongCapital() 
	{
		String tooLong = RandomStringUtils.randomAlphanumeric(36);
		testCountry.setCapital(tooLong);
	}
	
	@Test 
	public void testValidCapital() 
	{
		String[] values = {"Washington DC", "Tokyo", "Athens", "London"};
		for (String value : values)
		{
			testCountry.setCapital(value);
		}
	}
	
	@Test(expected = IllegalArgumentException.class)
	public void testNullContinent() 
	{
		testCountry.setContinent(null);
	}

	@Test(expected = IllegalArgumentException.class)
	public void testEmptyContinent() 
	{
		testCountry.setContinent("  ");
	}

	@Test(expected=IllegalArgumentException.class)
	public void testTooLongContinent() 
	{
		String tooLong = RandomStringUtils.randomAlphanumeric(21);
		testCountry.setContinent(tooLong);
	}
	
	@Test 
	public void testInvalidContinent() 
	{
		String[] values = {"America", "South Asia", "India", "Home"};
		for (String value : values)
		{
			try 
			{
				testCountry.setContinent(value);
				fail("the setContinent() method accepted an invalid value: " + value);
			} 
			catch (IllegalArgumentException oops) 
			{
				String expected = "continent must be a valid name";
				String actual = oops.getMessage();
				assertEquals(expected, actual);
			}
		}
	}
	
	@Test 
	public void testValidContinent() 
	{
		String[] values = {"Washington DC", "Tokyo", "Athens", "London"};
		for (String value : values)
		{
			testCountry.setCapital(value);
		}
	}
	
	@Test 
	public void testToString() 
	{
		String expected = "Country [code=ABW, name=Aruba, "
				+ "capital=Oranjestad, continent=North America]";
		String actual = testCountry.toString();
		assertEquals(expected, actual);
	}
	
	@Test public void testHashCode()
	{
	    Country sample = new Country();
	    int expected = sample.hashCode();
	    int actual = testCountry.hashCode();
	    assertEquals(expected, actual);
	}
	
	@Test public void testEquals()
    {
		Country sample = new Country();
        assertTrue(testCountry.equals(sample));
    }
	
	@Test public void testNotEquals()
    {
		Country sample = new Country();
        sample.setName("Japan");
        assertFalse(testCountry.equals(sample));
        
        Object junk = new Object();
        assertFalse(testCountry.equals(junk));
    }
}
